%% Contest visualization
%% Introduction
% This paper contains some observations that can be gleaned from the
% <http://www.mathworks.com/contest/jumping/rules.html Peg
% Solitare contest> result dataset. It
% was inspired by the now finished 
% <http://www.mathworks.com/contest/datavis/rules.html Data Visualization contest>.
% To be clear, _this is not an entry in the contest,_
% since it is too late.
% 
% Most of the entrants concentrated on analyzing data closely associated
% with authors. Here I consider some other data to demonstrate a few
% analysis and visualization methods.
%
% You can reproduce this paper by running the companion script file *vizcon.m* with the
% data file *contest_data.mat* in the same directory. I set Include Code
% in the Publish Configuration to false so that the flow is not interrupted.
% You can set it to true
% so that the code will be interspersed with the markup.
%% Acquire the data.

if ~exist('contest_data.mat','file')  
    warning (['This is a followup to the MATLAB programming contest ' ...
        '(http://www.mathworks.com/contest/datavis/home.html). ' ...
        'Please load the contest data and unzip it to place ' ...
        'contest_data.mat on your MATLAB path.'])  
    web(['http://www.mathworks.com/matlabcentral/fileexchange' ...
        '/23509?controller=file_infos&download=true'])  
end
%%
% Extract some variables of obvious interest (score, result, cpu_time)
% from the structure.
load contest_data
score = [d.score];
result = [d.result];
cpu_time = [d.cpu_time];
%% Look for data issues.
% The Workspace Browser shows NaN values for cpu_time and result as well as
% Inf values for cpu_time. 
%
% <<WorkBrows.png>>
% 
%%
% According the the contest rules, run times
% greater than 180 resulted in rejection, so these nonfinite values
% probably are caused by cpu_time NaN values. 
%
% We can see this in the distribution of times.
badFraction = sum(isnan(cpu_time))/numel(cpu_time);
plot(sort(cpu_time), 'LineWidth',2)
text(0.1*numel(cpu_time), 0.8*max(cpu_time), ...
    [int2str(100*badFraction) ' % of the entries were over the time limit.'])
xlabel('Sorted entry')
ylabel('CPU time')
title('Distribution of CPU times')
%%
% Interestingly the rejection rate (fraction of entries with run time
% greater than 180) is approximately constant over the
% contest duration.
% The rejected entries would need
% special treatment for further analysis.
% For now discard them and proceed with the valid entries.
fullD = d;
nWindowPoints = 100;
smooth = filter(ones(1,nWindowPoints)/nWindowPoints,1,[fullD.passed]);
smooth(1:nWindowPoints) = nan;
plot(smooth, 'LineWidth', 2)
ylim([0 1])
ylabel('Fraction of passing entries')
xlabel('Entry number')
title('Rejection trend')
%%
index = isnan(cpu_time);
d(index) = [];
score = [d.score];
result = [d.result];
cpu_time = [d.cpu_time];
complexity = [d.maxComplexity];
%
% Scale data for convenience.
scaledScore = score/1000;
scaledResult = result/1000;
%
% Keep track of the winner, second and third places.
% Note that low scores, results and CPU times are better!
[winningScore, winningIndex] = min(scaledScore);
winningResult = scaledResult(winningIndex);
winningTime = cpu_time(winningIndex);
[sortedScore, sortingIndex] = sort(scaledScore);
secondPlaceScore = scaledScore(sortingIndex(2));
thirdPlaceScore = scaledScore(sortingIndex(3));
secondPlaceResult = scaledResult(sortingIndex(2));
thirdPlaceResult = scaledResult(sortingIndex(3));
secondPlaceTime = cpu_time(sortingIndex(2));
thirdPlaceTime = cpu_time(sortingIndex(3));

%% Look at the distribution of scores.
% Most entries had almost the same score. Note that the lowest score is the
% winner! 
plot(sort(scaledScore), 'LineWidth', 2)
text(0.15*numel(scaledScore), 0.75*max(scaledScore), ...
    ['The worst to best ratio is more than 7.'])
xlabel('Sorted entry')
ylabel('score/1000')
title('Distribution of scores')
%% __Dependence on single parameters.
% The form of the scoring equation is provided in the contest rules. Let's
% look at how the computed score depends on the terms for the actual data.
% We see that the score depends strongly on result, somewhat on CPU time
% and very little on complexity.
subplot(1,3,1)
plot(scaledResult, scaledScore, '.')
xlim([0 200])
xlabel('result/1000')
ylabel('score/1000')
subplot(1,3,2)
plot(cpu_time, scaledScore, '.')
xlabel('CPU time')
title('Contest scoring parameters', 'FontWeight', 'Bold')
subplot(1,3,3)
plot(complexity, scaledScore, '.')
xlim([0 60])
xlabel('complexity')
%% __Parameter associations.
% Using brushing and linking we can see that the high scores with low
% results are caused by their large CPU times.
% The usual way to brush is by hand. This plot could be produced
% programmatically using *brush* and *linkdata*. I'll spare you the details.
%
% <<brush.png>>
%
%% __Dependence on the 2 main parameters.
% We can look into points very near the best score to see how score depends on
% the combination of result and cpu_time. This plot only shows data for
% scores less than 3950.
%
% Many of the entries had similar CPU times and very similar result
% numbers.
% A modest change to the result and cpu_time coefficients in the 
% scoring equation could have changed the finish order.
clf
index = scaledScore<3.95;
selectedScore = scaledScore(index);
selectedResult = scaledResult(index);
selectedTime = cpu_time(index);
scatter(selectedResult, selectedTime, 25, selectedScore)
hold on
hg = plot(winningResult, winningTime, 'x',  ...
    secondPlaceResult, secondPlaceTime, 'x',  ...
    thirdPlaceResult, thirdPlaceTime, 'x', 'MarkerSize', 15);
legend(hg, {'Best Score'; 'Second place'; 'Third place'})
hold off
title('2D score dependence')
xlabel('result/1000')
ylabel('cpu time')
%% Look into other associations.
% It is interesting to see if other parameters in the dataset are
% associated with low scoring entries.
%% __Number of characters
% Most entries had a smaller number of characters than the winner. It looks
% like terseness may not be such a good idea.
mlintCount = [d.mlintMsgCount];
charCount = [d.charCount];
plot(charCount, scaledScore, '.')
hold on
hg = plot(charCount(winningIndex), winningScore, 'rx', 'MarkerSize', 15);
hold off
legend(hg, 'Winner')
xlabel('Number of characters')
ylabel('score/1000')
%% __M-Lint messages
% Many of the entries had a smaller number of messages than the winner. It
% appears that some M-Lint warnings are not so urgent.
plot(mlintCount, scaledScore, '.')
hold on
hg = plot(mlintCount(winningIndex), winningScore, 'rx', 'MarkerSize', 15);
hold off
legend(hg, 'Winner')
xlabel('Number of M-Lint messages')
ylabel('score/1000')
%% __Complexity
% We already saw that score does not depend very much on complexity.
% The result value also does not depend on complexity.
% The cpu_time depends weakly if
% at all on complexity. Aside from these performance related observations, 
% code complexity can have a big impact on readability, which is
% quite important in real applications.
subplot(2,1,1)
plot(complexity, scaledResult,'.')
ylabel('result/1000')
subplot(2,1,2)
plot(complexity, cpu_time,'.')
xlabel('Complexity')
ylabel('CPU time')
%% __Time of day
% Daytime entries typically have better scores than than night entries. Of
% course it's always daytime somewhere.
darkScores = scaledScore(logical([d.darkness]));
darkMed = median(darkScores);
twiScores = scaledScore(logical([d.twilight]));
twiMed = median(twiScores);
dayScores = scaledScore(logical([d.daylight]));
dayMed = median(dayScores);
disp('Median ambient light scores')
disp(['Darkness: ' sprintf('%4.1f', darkMed)])
disp(['Twilight: ' sprintf('%4.1f', twiMed)])
disp(['Daylight: ' sprintf('%4.1f', dayMed)])
%%
% We can see this more completely by looking at the distributions of
% entries in a quantile plot. The scores of the darkness entries are always 
% the worst of the three. 
% Twilight scores are usually worse than daytime scores.
nDark = numel(darkScores);
nTwi = numel(twiScores);
nDay = numel(dayScores);
dayScale = (1:nDay)/nDay;
twiScale = (1:nTwi)/nTwi;
darkScale = (1:nDark)/nDark;

clf
plot(darkScale, sort(darkScores), twiScale, sort(twiScores), dayScale,...
    sort(dayScores), 'LineWidth', 2)
legend({'Darkness'; 'Twilight'; 'Daylight'}, 'Location', 'northwest')
ylabel('score/1000')
xlabel('f-value')
title('Distribution of scores by time of day')

%% Appendix
% I have included 2 figures that are not generated by this m-file. This is
% against the contest rules, but then I am not entering the contest.
%
% This file does not have a traditional H1 line worded for use with
% lookfor. Publishing calls for a title, so get used to using lookfor -all.
% Similarly initial comments that publish well may look different from the
% help block that we are used to. They should still contain the required
% information and be placed on contiguous % lines, but the arrangement may
% be different.

